@@ -3,6 +3,7 @@ from typing import Iterable, Optional, Sequence, Union
 
 from docx import Document
 from docx.shared import Inches, Pt
+from docx.text.parfmt import ParagraphFormat
 from pandas.core.frame import DataFrame
 from tqdm import tqdm
 
@@ -19,20 +20,16 @@ logging.basicConfig(
 
 
 class ReportContent:
-    """Creates objects that analyse data, and stores the results as textual
-    summaries and graphs.
+    """Analyses data, then prepares textual summaries and graphs.
 
-    Parameters
-    ----------
-    data : Iterable
-        The data to analyse.
-    target_variable : Optional[Union[str, int]]
-        The column to use to group values and color-code graphs.
-    title : str, optional
-        The title to assign the report, by default "Exploratory Data Analysis
-        Report"
-    graph_color : str, optional
-        The color to apply to the graphs, by default "cyan"
+    Args:
+        data (Iterable): The data to analyse.
+        title (str, optional): The title to assign the report. Defaults to
+            "Exploratory Data Analysis Report".
+        graph_color (str, optional): The color to apply to the graphs.
+            Defaults to "cyan".
+        target_variable (Optional[Union[str, int]], optional): The column to
+            use to group values. Defaults to None.
     """
 
     def __init__(
@@ -59,13 +56,10 @@ class ReportContent:
         self.bivariate_summaries = self._get_bivariate_summaries()
 
     def _get_introductory_summary(self) -> str:
-        """Get a brief overview of the number of rows and the nature of
-        columns.
+        """Get an overview of the number of rows and the nature of columns.
 
-        Returns
-        -------
-        str
-            An overview of the input data.
+        Returns:
+            str: Introduction
         """
         num_rows, num_cols = self.variables.data.shape
 
@@ -92,30 +86,36 @@ class ReportContent:
 
         return f"The dataset consists of {rows} and {cols}{numeric}."
 
-    def _describe_variable(self, name: str) -> dict:
+    def _describe_variable(self, variable: Variable) -> dict:
+        """Get summary statistics for a variable.
 
-        # Perform univariate analysis using the Variable class
-        var = Variable(self.variables.data[name])
+        Args:
+            variable (Variable): The data to analyse.
 
-        if var.num_unique == 1:
+        Returns:
+            dict: Summary statistics
+        """
+        var = variable.contents
+        if variable.num_unique == 1:
             unique_vals = "1 unique value"
         else:
-            unique_vals = f"{var.num_unique:,} unique values"
+            unique_vals = f"{variable.num_unique:,} unique values"
 
         return {
             "description": (
-                f"{name.capitalize()} is a {var.var_type} variable with"
-                f" {unique_vals}. {var.missing} of its values are missing."
+                f"{variable.name.capitalize()} is a {variable.var_type} "
+                f"variable with {unique_vals}. {variable.missing} of its "
+                "values are missing."
             ),
             "graphs": PlotVariable(
-                variable=var,
+                variable,
                 graph_color=self.GRAPH_COLOR,
                 hue=self.TARGET_VARIABLE,
             ).graphs,
-            "statistics": var.statistics,
-            "most_common_items": (
-                var.most_common_items
-                if hasattr(var, "most_common_items")
+            "statistics": var._get_summary_statistics().to_frame(),
+            "normality_tests": (
+                var._test_for_normality()
+                if variable.var_type == "numeric"
                 else None
             ),
         }
@@ -123,19 +123,18 @@ class ReportContent:
     def _get_variable_descriptions(self) -> dict:
         """Get brief descriptions of all columns present in the data.
 
-        Returns
-        -------
-        dict[str, str]
-            Summaries of columns present.
+        Returns:
+            dict: Summaries of columns present.
         """
         return {
-            col: self._describe_variable(col)
-            for col in tqdm(
-                self.variables.data.columns,
+            var.name: self._describe_variable(var)
+            for var in tqdm(
+                self.variables.iter_variables(),
                 bar_format="{desc}: {percentage:3.0f}%|{bar:35}| "
                 + "{n_fmt}/{total_fmt} features.",
                 desc="Univariate analysis",
                 dynamic_ncols=True,
+                total=self.variables.data.shape[1],
             )
         }
 
@@ -143,10 +142,8 @@ class ReportContent:
         """Get descriptions of the nature of correlation between numeric
         column pairs.
 
-        Returns
-        -------
-        dict[str, str]
-            Correlation info.
+        Returns:
+            dict: Correlation info.
         """
         if hasattr(self.variables, "var_pairs"):
             return {
@@ -172,26 +169,22 @@ class ReportDocument(ReportContent):
 
     #. An *Overview* of the data and its features.
     #. *Univariate Analysis*: Summary statistics and graphs for each feature.
-    #. *Bivariate Analysis*: Pairwise comparisons of all numerical features.
+    #. *Bivariate Analysis*: Pairwise comparisons of numerical features.
 
     .. _python-docx: https://python-docx.readthedocs.io/en/latest/
 
-    Parameters
-    ----------
-    data : Iterable
-        The data to analyse.
-    target_variable : Optional[Union[str, int]]
-        The column to use to group values and color-code graphs.
-    title : str, optional
-        The title to assign the report, by default "Exploratory Data Analysis
-        Report"
-    graph_color : str, optional
-        The color to apply to the graphs, by default "cyan"
-    output_filename : str, optional
-        The file name or path to save the document to, by default
-        "eda-report.docx"
-    table_style : str, optional
-        The style to apply to the tables created, by default "Table Grid"
+    Args:
+        data (Iterable): The data to analyse.
+        title (str, optional): The title to assign the report. Defaults to
+            "Exploratory Data Analysis Report".
+        graph_color (str, optional): The color to apply to the graphs.
+            Defaults to "cyan".
+        target_variable (Optional[Union[str, int]], optional): The column to
+            use to group values. Defaults to None.
+        output_filename (str, optional): The file name or path to save the
+            document to. Defaults to "eda-report.docx".
+        table_style (str, optional): The style to apply to the tables created.
+            Defaults to "Table Grid".
     """
 
     def __init__(
@@ -215,17 +208,17 @@ class ReportDocument(ReportContent):
         self.document = Document()
         self._get_report()
 
-    def _get_report(self):
+    def _get_report(self) -> None:
         """Calculate summary statistics, plot graphs, and save the results as
         a .docx file.
         """
         self._create_title_page()
-        self._get_variable_info()
+        self._get_univariate_analysis()
 
         if hasattr(self.variables, "var_pairs"):
             self._get_bivariate_analysis()
 
-        self._save_file()
+        self._to_file()
         logging.info(f"Done. Results saved as {self.OUTPUT_FILENAME!r}")
 
     def _create_title_page(self) -> None:
@@ -233,17 +226,33 @@ class ReportDocument(ReportContent):
         self.document.add_heading(self.TITLE, level=0)
 
         self.document.add_paragraph(self.intro_text)
-        self.document.add_paragraph()
 
         self._get_numeric_overview_table()
         self._get_categorical_overview_table()
+        self.document.add_page_break()
+
+    def _format_heading_spacing(
+        self, format: ParagraphFormat, before: int = 21, after: int = 7
+    ) -> None:
+        """Set the spacing above or below a heading.
+
+        Args:
+            format (ParagraphFormat): The heading's format.
+            before (int, optional): Size of spacing above the heading in pt.
+                Defaults to 21.
+            after (int, optional): Size of spacing below the heading in pt.
+                Defaults to 7.
+        """
+        format.space_before = Pt(before)
+        format.space_after = Pt(after)
 
     def _get_numeric_overview_table(self) -> None:
         """Create a table with an overview of the numeric features present."""
         if self.variables.numeric_cols is not None:
-            self.document.add_heading("Overview of Numeric Features", level=2)
-            self.document.add_paragraph()
-
+            heading = self.document.add_heading(
+                "Overview of Numeric Features", level=1
+            )
+            self._format_heading_spacing(heading.paragraph_format)
             # count | mean | std | min | 25% | 50% | 75% | max
             self._create_table(
                 data=self.variables.numeric_stats,
@@ -258,11 +267,10 @@ class ReportDocument(ReportContent):
         present.
         """
         if self.variables.categorical_cols is not None:
-            self.document.add_heading(
-                "Overview of Categorical Features", level=2
+            heading = self.document.add_heading(
+                "Overview of Categorical Features", level=1
             )
-            self.document.add_paragraph()
-
+            self._format_heading_spacing(heading.paragraph_format)
             # column-name | count | unique | top | freq | relative freq
             self._create_table(
                 data=self.variables.categorical_stats,
@@ -272,30 +280,44 @@ class ReportDocument(ReportContent):
                 style="Normal Table",
             )
 
-    def _get_variable_info(self) -> None:
+    def _get_univariate_analysis(self) -> None:
         """Get a brief introduction, summary statistics, and graphs for each
         individual variable.
         """
-        self.document.add_heading("A. Univariate Analysis", level=1)
-
+        univariate_heading = self.document.add_heading(
+            "1. Univariate Analysis", level=1
+        )
+        self._format_heading_spacing(
+            univariate_heading.paragraph_format, before=0, after=0
+        )
         for idx, var_name in enumerate(self.variable_descriptions, start=1):
             var_info = self.variable_descriptions[var_name]
 
-            self.document.add_heading(f"{idx}. {var_name}".title(), level=2)
+            heading = self.document.add_heading(
+                f"1.{idx}. {var_name}".title(), level=2
+            )
+            self._format_heading_spacing(
+                heading.paragraph_format, before=20, after=5
+            )
             self.document.add_paragraph(var_info["description"])
 
             self.document.add_heading("Summary Statistics", level=4)
             self._create_table(var_info["statistics"], column_widths=[2.5, 2])
 
-            # Add most-common-items table, if present
-            if var_info.get("most_common_items") is not None:
-                self.document.add_heading("Most Common Values", level=4)
-                self._create_table(
-                    var_info["most_common_items"], column_widths=(2.5, 2)
-                )
-
             for graph in var_info["graphs"].values():
-                self.document.add_picture(graph, width=Inches(5.4))
+                self.document.add_picture(graph, width=Inches(4.4))
+
+            if (norm_tests := var_info["normality_tests"]) is not None:
+
+                self.document.add_heading("Tests for Normality", level=4)
+                # type | p-value | conclusion
+                self._create_table(
+                    data=norm_tests,
+                    header=True,
+                    column_widths=(2.2, 1, 2),
+                    font_size=8.5,
+                    style="Normal Table",
+                )
 
         self.document.add_page_break()
 
@@ -304,10 +326,11 @@ class ReportDocument(ReportContent):
         variables.
         """
         bivariate_heading = self.document.add_heading(
-            "B. Bivariate Analysis (Correlation)", level=1
+            "2. Bivariate Analysis (Correlation)", level=1
+        )
+        self._format_heading_spacing(
+            bivariate_heading.paragraph_format, before=0
         )
-        bivariate_heading.paragraph_format.space_before = Pt(0)
-        self.document.add_paragraph()
 
         self.document.add_picture(
             self.multivariate_graphs["correlation_heatmap"], width=Inches(6.7)
@@ -315,15 +338,17 @@ class ReportDocument(ReportContent):
         self.document.add_page_break()
 
         for idx, var_pair in enumerate(self.bivariate_summaries, start=1):
-            self.document.add_heading(
-                f"{idx}. {var_pair[0]} vs {var_pair[1]}".title(), level=2
+            heading = self.document.add_heading(
+                f"2.{idx} {var_pair[0]} vs {var_pair[1]}".title(), level=2
+            )
+            self._format_heading_spacing(
+                heading.paragraph_format, before=20, after=5
             )
             self.document.add_paragraph(self.bivariate_summaries[var_pair])
             self.document.add_picture(
                 self.multivariate_graphs["scatterplots"][var_pair],
                 width=Inches(6),
             )
-            self.document.add_paragraph()
 
             # Add a page break after every 2 pairs
             if idx % 2 == 0:
@@ -338,22 +363,19 @@ class ReportDocument(ReportContent):
         style: Optional[str] = None,
         header: bool = False,
     ) -> None:
-        """Generates a table for the supplied `data` with
+        """Generates a table for the supplied ``data``.
 
-        Parameters
-        ----------
-        data : DataFrame
-            The data to tabulate.
-        column_widths : Sequence, optional
-            The desired number and widths of columns, by default ().
-        font_face : str, optional
-            The font typeface for cell text, by default "Courier New".
-        font_size : float, optional
-            The font size for cell text, by default 10.
-        style : Optional[str], optional
-            A Word table style, by default None.
-        header : bool, optional
-            Flags whether the first row is a header, by default False.
+        Args:
+            data (DataFrame): The data to tabulate.
+            column_widths (Sequence, optional): Column specifications.
+                Defaults to ().
+            font_face (str, optional): Font typeface for cell text. Defaults
+                to "Courier New".
+            font_size (float, optional): Font size. Defaults to 10.
+            style (Optional[str], optional): A `Word` table style. Defaults to
+                None.
+            header (bool, optional): Whether or not to include column names.
+                Defaults to False.
         """
         if header:
             # Add a row of column labels
@@ -386,6 +408,11 @@ class ReportDocument(ReportContent):
 
         self.document.add_paragraph()
 
-    def _save_file(self) -> None:
-        """Save the document as a file."""
+    def _to_file(self) -> None:
+        """Save the report as a file."""
+        # Set page margins
+        for section in self.document.sections:
+            section.left_margin = Inches(1.2)
+            section.right_margin = Inches(1.2)
+
         self.document.save(self.OUTPUT_FILENAME)
