@@ -1,15 +1,17 @@
 import logging
 from collections.abc import Iterable
 from itertools import combinations
+from types import GeneratorType
 from typing import Sequence, Union
 
 from pandas.core.frame import DataFrame
 
+from eda_report.univariate import Variable
 from eda_report.validate import validate_multivariate_input
 
 
 class MultiVariable:
-    """Creates objects that analyse data with *multiple columns*.
+    """Defines objects that analyse two-dimensional datasets.
 
     Input data is held as a :class:`pandas.DataFrame` in order to leverage
     pandas_ built-in statistical methods, as well as functions
@@ -18,37 +20,34 @@ class MultiVariable:
     .. _pandas: https://pandas.pydata.org/
     .. _SciPy ecosystem: https://www.scipy.org/
 
-    Parameters
-    ----------
-    data : Iterable
-        The data to analyse.
+    .. note::
+       Not meant to be used directly: use the :func:`eda_report.summarize`
+       function instead.
 
-    Example
-    -------
-    .. literalinclude:: examples.txt
-       :lines: 87-116
+    Args:
+        data (Iterable): The data to analyse.
+
+    Example:
+        .. literalinclude:: examples.txt
+           :lines: 84-112
     """
 
     def __init__(self, data: Iterable) -> None:
-        """Initialise an instance of
-        :class:`~eda_report.multivariate.MultiVariable`.
-        """
+
         self.data = validate_multivariate_input(data)
 
         #: :class:`~pandas.DataFrame`: The *numeric columns* present.
         self.numeric_cols = self._select_cols("number")
 
-        #: :class:`~pandas.DataFrame`: The *categorical columns*. Please note
-        #: that **boolean** and **datetime** features are also
-        #: **considered categorical** in this context.
+        #: :class:`~pandas.DataFrame`: The *categorical columns* present.
         self.categorical_cols = self._select_cols("object", "bool")
 
         #: :class:`~pandas.DataFrame`: Summary statistics for numeric columns.
-        self.numeric_stats = self._compute_numeric_summary_statistics()
+        self.numeric_stats = self._get_numeric_summary_statistics()
 
         #: :class:`~pandas.DataFrame`: Summary statistics for categorical
         #:  columns.
-        self.categorical_stats = self._compute_categorical_summary_statistics()
+        self.categorical_stats = self._get_categorical_summary_statistics()
 
         #: :class:`~pandas.DataFrame`: Pearson correlation coefficients for the
         #: *numeric columns*.
@@ -60,83 +59,79 @@ class MultiVariable:
         self._get_bivariate_analysis()
 
     def __repr__(self) -> str:
-        """Defines the string representation of :class:`Multivariable`.
+        """Get the string representation of :class:`Multivariable`.
 
-        Returns
-        -------
-        str
-            The string representation of the ``MultiVariable`` instance.
+        Returns:
+            str: The string representation of the ``MultiVariable`` instance.
         """
-        # Get a list of numeric features
-        numeric_cols = "" if self.numeric_cols is None else self.numeric_cols
-        # Get a list of categorical features
-        categoric_cols = (
-            "" if self.categorical_cols is None else self.categorical_cols
-        )
-        correlation_description = (
-            self._corr_description
-            if hasattr(self, "_corr_description")
-            else "N/A"
-        )
+        if self.data.shape[1] == 1:
+            return str(Variable(self.data.squeeze()).contents)
+
+        if self.numeric_cols is None:
+            numeric_info = numeric_stats = ""
+        else:
+            numeric_info = f"Numeric features: {', '.join(self.numeric_cols)}"
+            numeric_stats = (
+                "\n\t  Summary Statistics (Numeric features)\n"
+                "\t  -------------------------------------\n"
+                f"{self.numeric_stats}"
+            )
+
+        if self.categorical_cols is None:
+            categorical_info = categorical_stats = ""
+        else:
+            categorical_info = (
+                f"Categorical features: {', '.join(self.categorical_cols)}"
+            )
+            categorical_stats = (
+                "\n\t  Summary Statistics (Categorical features)\n"
+                "\t  -----------------------------------------\n"
+                f"{self.categorical_stats}"
+            )
+        if hasattr(self, "_corr_description"):
+            correlation_description = (
+                "\n\t  Bivariate Analysis (Correlation)\n"
+                "\t  --------------------------------\n"
+                f"{self._corr_description}"
+            )
+        else:
+            correlation_description = ""
+
         return "\n".join(
             [
                 "\t\t\tOVERVIEW",
                 "\t\t\t========",
-                f"Numeric features: {', '.join(numeric_cols)}",
-                f"Categorical features: {', '.join(categoric_cols)}",
-                "\t\t\t  ***",
-                "\t  Summary Statistics (Numeric features)",
-                "\t  -------------------------------------",
-                f"{self.numeric_stats if any(numeric_cols) else 'N/A'}",
-                "\t\t\t  ***",
-                "\t  Summary Statistics (Categorical features)",
-                "\t  -----------------------------------------",
-                f"{self.categorical_stats if any(categoric_cols) else 'N/A'}",
-                "\t\t\t  ***",
-                "\t  Bivariate Analysis (Correlation)",
-                "\t  --------------------------------",
+                f"{numeric_info}",
+                f"{categorical_info}",
+                f"{numeric_stats}",
+                f"{categorical_stats}",
                 f"{correlation_description}",
             ]
         )
 
-    def describe(self) -> None:
-        """Display summary statistics for both numerical and categorical
-        columns.
-        """
-        if self.numeric_stats is None:
-            print("\nThere are no numeric columns.\n")
-        else:
-            print(
-                "\n\tSummary Statistics (Numeric columns):\n\n",
-                self.numeric_stats,
-            )
+    def iter_variables(self) -> GeneratorType:
+        """Iterate through all the variables present in alphabetic order.
 
-        if self.categorical_stats is None:
-            print("\nThere are no categorical columns.\n")
-        else:
-            print(
-                "\n\tSummary Statistics (Categorical columns):\n\n",
-                self.categorical_stats,
-            )
+        Yields:
+            GeneratorType: Variables
+        """
+        for name, data in self.data.sort_index(axis=1).items():
+            yield Variable(data=data, name=name)
 
     def _select_cols(self, *dtypes: Sequence[str]) -> Union[DataFrame, None]:
         """Get a DataFrame including only the specified ``dtypes``.
 
-        Returns
-        -------
-        Union[DataFrame, None]
-            A dataframe with the desired data types.
+        Returns:
+            Union[DataFrame, None]: A dataframe with the desired data types.
         """
         selected_cols = self.data.select_dtypes(include=dtypes)
         return selected_cols if selected_cols.shape[1] > 0 else None
 
-    def _compute_numeric_summary_statistics(self) -> DataFrame:
-        """Get descriptive statistics for numeric columns.
+    def _get_numeric_summary_statistics(self) -> Union[DataFrame, None]:
+        """Compute descriptive statistics for numeric columns.
 
-        Returns
-        -------
-        DataFrame
-            Numeric summary statistics.
+        Returns:
+            Union[DataFrame, None]: Numeric summary statistics.
         """
         if self.numeric_cols is not None:
             numeric_stats = self.numeric_cols.describe().T
@@ -151,13 +146,13 @@ class MultiVariable:
         else:
             return None
 
-    def _compute_categorical_summary_statistics(self) -> DataFrame:
-        """Get summary descriptive statistics for categorical columns.
+    def _get_categorical_summary_statistics(
+        self,
+    ) -> Union[DataFrame, None]:
+        """Compute descriptive statistics for categorical columns.
 
-        Returns
-        -------
-        DataFrame
-            Categorical summary statistics.
+        Returns:
+            Union[DataFrame, None]: Categorical summary statistics.
         """
         if self.categorical_cols is not None:
             categorical_stats = self.categorical_cols.describe().T
@@ -169,28 +164,29 @@ class MultiVariable:
             return None
 
     def _get_correlation(self) -> Union[DataFrame, None]:
-        """Get a DataFrame of the correlation coefficients for numeric
-        columns.
+        """Get the Pearson correlation coefficients for numeric columns.
 
-        Returns
-        -------
-        Union[DataFrame, None]
-            A dataframe of Pearson correlation coefficients, or None.
+        Returns:
+            Union[DataFrame, None]: Correlation coefficients
         """
-        return None if self.numeric_cols is None else self.numeric_cols.corr()
-
-    def _get_variable_pairs(self) -> None:
-        """Get a list of unique pairings of the numeric columns."""
-        self.var_pairs = set(combinations(self.correlation_df.columns, r=2))
+        if self.numeric_cols is None:
+            return None
+        else:
+            unique_ratio = self.numeric_cols.nunique() / len(self.data)
+            cols_to_compare = [
+                col for col, ratio in unique_ratio.items() if ratio > 0.05
+            ]
+            if len(cols_to_compare) >= 2:
+                return self.numeric_cols[cols_to_compare].corr()
+            else:
+                return None
 
     def _quantify_correlation(self, var1: str, var2: str) -> None:
         """Explain the nature and magnitude of correlation between column
         pairs.
 
-        Parameters
-        ----------
-        var1, var2 : str
-            Numeric column labels.
+        Args:
+            var1, var (str): Numeric column labels.
         """
         correlation = self.correlation_df.loc[var1, var2]
         nature = " positive" if correlation > 0 else " negative"
@@ -214,19 +210,15 @@ class MultiVariable:
             (var1, var2)
         ] = f"{strength}{ nature} correlation ({correlation:.2f})"
 
-    def _compare_variable_pairs(self) -> None:
-        """Get a brief summary of the nature of correlation between pairs of
-        numeric columns.
-        """
-        self._get_variable_pairs()
-
-        for var1, var2 in self.var_pairs:
-            self._quantify_correlation(var1, var2)
-
     def _get_bivariate_analysis(self) -> None:
         """Compare numeric column pairs."""
-        if self.numeric_cols is not None and self.numeric_cols.shape[1] > 1:
-            self._compare_variable_pairs()
+        if self.correlation_df is not None:
+            self.var_pairs = set(
+                combinations(self.correlation_df.columns, r=2)
+            )
+            for var1, var2 in self.var_pairs:
+                self._quantify_correlation(var1, var2)
+
             self._corr_description = "\n".join(
                 [
                     f"{var_pair[0]} & {var_pair[1]} --> {corr_description}"
@@ -238,7 +230,7 @@ class MultiVariable:
             )
         else:
             logging.warning(
-                "Skipped Bivariate Analysis: "
-                "Not enough numeric variables to compare."
+                "Skipped Bivariate Analysis: There are less than 2 numeric "
+                "variables having > 5% unique values."
             )
             return None
